local class = require 'lib.middleclass'

Bomb = class('Bomb')

function Bomb:initialize(_x, _y, _speedx, _speedy, _angle, origin, modifiers) --originating entity
	
	self.id = math.random(1,100000) -- for testing purposes
	self.h = 5
	self.w = 5
	self.radius = 10
	self.x = _x
	self.y = _y
	self.angle = _angle
	self.firingspeed = 700
	self.lifespan = 2
	self.damage = 100
	self.firingtimer = 3
	self.bounces = 1
	self.origin = origin
	
	prop_max = 	{ 						-- maximum modifer bonus
							fs = 400,
							ls = 3,
							dmg = 100,
							ft = 2.5,
							b = 3
						}
	
	if (modifiers ~= nil) then -- not sure i need to check 
		self.firingspeed = self.firingspeed + modifiers.fs
		self.lifespan = self.lifespan + modifiers.ls
		self.damage = self.damage + modifiers.dmg
		self.firingtimer = self.firingtimer - modifiers.ft
		self.bounces =  self.bounces + modifiers.b
	end

	-- these need to be assigned after modifiers 
	self.speedX = _speedx + self.firingspeed  * math.cos(_angle)
	self.speedY = _speedy + self.firingspeed  * math.sin(_angle)
	
end

function Bomb:draw(sh) -- add getcolor setcolor?
	
	if self.origin == "player" then
		love.graphics.setColor(30,144,255)		
	else
		love.graphics.setColor(220,20,60) -- enemy
	end
		
		
	sh:draw('fill')
	
end

function Bomb:update(sh, dt)
			
	local sx, sy = sh:center()
	local s = self
	
	sx = sx + s.speedX * dt
	sy = sy + s.speedY * dt
	
	s.lifespan = s.lifespan - dt	
	
	sh:move(s.speedX * dt, s.speedY * dt)
		
end

function Bomb:getHit(sh, damage)
	self:explode(sh)
end

function Bomb:explode(sh)  --  shape	
	
	local sx, sy = sh:center()
	game:spawnExplosion(sx, sy, 80) 
	self.lifespan = 0
	
end

function Bomb:die(i, sh) -- we need index to remove it from bullet table.
									-- if you want a bullet to die outside of main.lua update bullets loop, use self.lifespan=0

	Collider:remove(sh)
	table.remove(bullets, i)
	--print ('removed bullet ' .. i )
	
end

function Bomb:resolveCollision(sh, dx, dy)
	
	-- move shape to resolve collision
	sh:move(dx, dy) 

	if self.bounces > 0 then
		-- reverse speed on the axis with the largest distance to travel to resolve the collision
		-- creates a rebound
		if (math.abs(dx) > math.abs(dy)) then
			self.speedX = -self.speedX
		else 
			self.speedY = -self.speedY
		end
		self.bounces = self.bounces - 1
	else
		self.lifespan = 0
	end
	
		
end

function Bomb:resolveRepel(sh, dx, dy, repel_shape) --
local force = repel_shape.instance.force
local sx, sy = sh:center()
local rx, ry = repel_shape:center()
local repel_angle = math.angle(rx, ry, sx, sy)

	-- move shape to resolve collision
	sh:move(dx, dy) 

	-- rebound 
	if (math.abs(dx) > math.abs(dy)) then
		-- if you are moving in a direction, being hit in the same direction should not cause you to reverse direction
		if dx < 0 and self.speedX > 0 or dx > 0 and self.speedX < 0 then 
			self.speedX = -self.speedX
		end
	else
		if dy < 0 and self.speedY > 0 or dy > 0 and self.speedY < 0 then
			self.speedY = -self.speedY
		end
	end
	
	
	self.speedX = self.speedX + force * math.cos(repel_angle)
	self.speedY = self.speedY + force * math.sin(repel_angle)

end

function Bomb:playSound()
	if sfx.explosion:isPlaying() then
		local newSource = sfx.explosion:clone()
		newSource:play()
	else
		sfx.explosion:play()
	end
end


return Bomb