local class = require 'lib.middleclass'


Enemy1 = class('Enemy1')

function Enemy1:initialize(_x, _y, _size) 

	self.name = "enemy"

	self.id = math.random(1,100000) -- for testing purposes
	
	local r_size = math.random(1,3)
	local new_size
	
	if r_size == 1 then
		new_size = 90
	elseif r_size == 2 then
		new_size = 60
	else
		new_size = 30
	end
	
	if (_size == nil) then
		self.h = new_size
		self.w = self.h
	else
		self.h = _size
		self.w = _size
	end 
	
	self.x = _x
	self.y = _y
	self.angle = 0
	self.firingspeed = 0
	self.maxspeed = math.random(250, 450)
	self.friction = 0.99
	self.speedX = math.random(-self.maxspeed, self.maxspeed)
	self.speedY = math.random(-self.maxspeed, self.maxspeed)
	
	local modifier = game:getStageModifier() 
	
	self.maxhealth = 100 * modifier -- add health based on stage modifier.	
	self.health = self.maxhealth
	
	self.spawntimermax = math.random(15,25)
	self.spawntimer = 0
	self.pointvalue = math.round(1000 / self.h) * modifier -- pointvalue based on size and stage modifier. smaller enemy1 are worth more; harder to target.
	
	-- FOR FUN : randomized weapon
	
	local w = math.random (1,10)
	local str_weapon
	
	if w < 2 then
		str_weapon = "bullet"
	elseif w < 8 then
		str_weapon = "bomb"
	else
		str_weapon = "spreadbomb"
	end
	
	self.weapon = Weapon(str_weapon, self.name) --Weapon("bullet", self.name)
	
	self.min_fire_range = 250
	self.max_fire_range = 650
	
end

function Enemy1:updateSpawningTimer(sh, dt) -- enemy1 spawns additional enemies intermittently (atm just enemy2)
	local sx, sy = sh:center()

	if self.spawntimer >= self.spawntimermax then
		-- spawn an enemy2 entity
		local newEntity = Entities("enemy2", sx, sy)
		newEntity:spawn()		
		self.spawntimer = 0
	else 
		self.spawntimer = self.spawntimer + dt
	end

end

function Enemy1:draw(sh)
		
	self:setColor()	
	
	local sx, sy = sh:center()
	local r,g,b = love.graphics.getColor()
	
	love.graphics.setColor(r,g,b,15)
	
	-- draw range radius
	local thickness = self.max_fire_range - self.min_fire_range
	love.graphics.setLineWidth( thickness )
	love.graphics.circle('line', sx, sy, self.min_fire_range + (thickness/2))
	love.graphics.setLineWidth(1)
	
	
	-- draw shape (self)
	love.graphics.setColor(r,g,b,255)
	sh:draw('fill')
		
	-- DRAW HEALTH BAR
	local percent = (self.health / self.maxhealth) * 100
	local lx, ly = sx - self.h/2, sy - self.h/2 - 20
	local lx2, ly2 = sx + self.h/2, ly
	local dist = math.dist(lx, ly, lx2, ly2)
	local line_dist = dist * (percent/100)
	love.graphics.line(lx, ly, lx + line_dist, ly2)
	
	-- PRINT SELECTED WEAPON
	love.graphics.setColor(255,255,255)
	local font_w = mainFont:getWidth(self.weapon.weapon_type) -- not sure if this detects scale too. seems to return half as wide as i sould expect.
	love.graphics.print(self.weapon.weapon_type, sx - font_w , ly - 50, 0, 2, 2)
	
	-- debug?
	--love.graphics.setColor(255,255,255)
	--love.graphics.print(tostring(self.weapon.firingtimer), sx - 100, sy -100, 0, 2, 2)
	--love.graphics.print(tostring(self.weapon.armed), sx - 100, sy + 100, 0, 2, 2)
	
	
end

function Enemy1:update(sh, dt)
					
					
		-- need a maxspeed becausae of repel increasing speed 
		if self.speedY > self.maxspeed then
			self.speedY = self.speedY * self.friction
		elseif self.speedY < (self.maxspeed * -1) then
			self.speedY = self.speedY * self.friction
		end
		if self.speedX > self.maxspeed then
			self.speedX = self.speedX * self.friction
		elseif self.speedX < (self.maxspeed * -1) then
			self.speedX = self.speedX * self.friction
		end


		
		sh:move(self.speedX * dt, self.speedY * dt)
		
		self.weapon:update(dt)
		
		local px, py = player1:center()
		local sx, sy = sh:center()
		
		local dist = math.dist(sx,sy,px,py) 
		
		if (dist < self.max_fire_range) and (dist > self.min_fire_range) then
		--print('try to attack')
							
			if self.health <= (self.maxhealth/2 + 1) and (self.health > 0) then -- enemy shouldn't attack after it's dead (self.health > 0)
				self:attack(sh) 				
			end
		end
		
		self:updateSpawningTimer(sh,dt)
	
end
  
function Enemy1:setColor()  

	local color = self:getColor()

	love.graphics.setColor(color.r, color.g, color.b)

end

function Enemy1:getColor()
	local health = self.health
	local maxhealth = self.maxhealth
	local color = {}
	
	if health <= (maxhealth/2 +1) then
		color.r = 240
		color.g = 230
		color.b = 140
	else
		color.r = 51
		color.g = 86
		color.b = 37
	end

	return color
end
  
function Enemy1:getHit(sh, damage) 
	self.health = self.health - damage
	
	if self.health <= (self.maxhealth/2+1) then
		self.weapon.armed = true -- arm weapon when at 50% life.
	end
end 
  
  function Enemy1:explode(sh)  --  shape	
	self.health = 0 -- ugh, he will explode in die() function.	
end
  
  
  
 function Enemy1:die(i, sh) -- index, shape
 
	player1.instance.score = player1.instance.score + self.pointvalue
 
	local x, y = sh:center()
	local size = self.h
	
	-- explode
	game:spawnExplosion(x, y, size, true)

	-- announce
	game:spawnAnnouncement(x - size / 2, y - size *2, "gold", tostring(self.pointvalue) .. " pts")
	
	
	
	-- remove enemy from table
	Collider:remove(sh)
	table.remove(entities, i)

	--if its larger than size 35, spawn 2 smaller ones
	if (size > 35) then
	
		local newEntity = Entities("enemy1", x, y, size - 30) -- this -30 is just a trick. i should create a function to getSize or something. 
		newEntity:spawn()
		newEntity:spawn()
	else
	
		-- enemy drops a modifier pickup
		local newEntity = Entities("pickup", x, y)
		newEntity:spawn()
		
	end
 
 end
 
 function Enemy1:resolveCollision(sh, dx, dy)

	-- move shape to resolve collision
	sh:move(dx, dy)
	
	-- rebound 
	if (math.abs(dx) > math.abs(dy)) then
		-- if you are moving in a direction, being hit in the same direction should not cause you to reverse direction
		if dx < 0 and self.speedX > 0 or dx > 0 and self.speedX < 0 then 
			self.speedX = -self.speedX
		end
	else
		if dy < 0 and self.speedY > 0 or dy > 0 and self.speedY < 0 then
			self.speedY = -self.speedY
		end
	end

end

function Enemy1:resolveRepel(sh, dx, dy, repel_shape) --
local force = repel_shape.instance.force
local sx, sy = sh:center()
local rx, ry = repel_shape:center()
local repel_angle = math.angle(rx, ry, sx, sy)

	-- move shape to resolve collision
	sh:move(dx, dy) 

	-- rebound 
	if (math.abs(dx) > math.abs(dy)) then
		-- if you are moving in a direction, being hit in the same direction should not cause you to reverse direction
		if dx < 0 and self.speedX > 0 or dx > 0 and self.speedX < 0 then 
			self.speedX = -self.speedX
		end
	else
		if dy < 0 and self.speedY > 0 or dy > 0 and self.speedY < 0 then
			self.speedY = -self.speedY
		end
	end
	
	
	self.speedX = self.speedX + force * math.cos(repel_angle)
	self.speedY = self.speedY + force * math.sin(repel_angle)

end


function Enemy1:attack(sh)

	
	local px, py = player1:center()
	local sx, sy = sh:center()
	
	local angle = math.angle(sx, sy, px, py)
	
	local points = {}
	points[0] = { x = sx, y = sy }
	self.weapon:fire(points, self.speedX, self.speedY, angle, self.name) -- FIRE
	
	
	
end
  
  
   
return Enemy1