local class = require 'lib.middleclass'


Enemy2 = class('Enemy2')

function Enemy2:initialize(_x, _y, _size) 

	self.name = "enemy"
	
	self.id = math.random(1,100000) -- for testing purposes
	
	local r_size = math.random(1,3)
	local new_size
	
	if r_size == 1 then
		new_size = 10
	elseif r_size == 2 then
		new_size = 20
	else
		new_size = 30
	end
	
	if (_size == nil) then
		self.h = new_size
		self.w = self.h
	else
		self.h = _size
		self.w = _size
	end 
	
	self.x = _x
	self.y = _y
	self.angle = 0
	self.firingspeed = 0
	self.rotatespeed = 2.5
	self.accelerationX = 0
	self.accelerationY = 0
	self.friction = 0.99
	self.thrust = math.random(300,600)
	self.maxspeed = math.random(500,1200)
	self.speedX = 0 --math.random(-350, 350)
	self.speedY = 0 --math.random(-350, 350)
	
	local modifier = game:getStageModifier() 
	
	-- this enemy doesn't split atm.
	self.maxhealth = new_size * modifier -- add health based on stage modifier (and for enemy2, spawn size)
	self.health = self.maxhealth
	
	self.pointvalue = math.round(1000 / self.h) * modifier
	
	-- FOR FUN : randomized weapon
	
	local w = math.random (1,10)
	local str_weapon
	
	if w < 2 then
		str_weapon = "bullet"
	elseif w < 8 then
		str_weapon = "bomb"
	else
		str_weapon = "spreadbomb"
	end
	
	self.weapon = Weapon(str_weapon, self.name) --Weapon("bullet", self.name)
	self.weapon.armed = true
		
	self.min_fire_range = 0
	self.max_fire_range = 800
	
	
	self.state = "hunting" -- none yet. this doesnt' do anything.
	self.path = nil
	
end

function Enemy2:draw(sh)
		
	self:setColor()	
	
	local sx, sy = sh:center()
	local r,g,b = love.graphics.getColor()
	
	love.graphics.setColor(r,g,b,15) 

	-- draw range radius
	local thickness = self.max_fire_range - self.min_fire_range
	love.graphics.setLineWidth( thickness )
	love.graphics.circle('line', sx, sy, self.min_fire_range + (thickness/2))
	love.graphics.setLineWidth(1)
	
	-- draw shape (self)
	love.graphics.setColor(r,g,b,255)
	sh:draw('fill')
		
	-- DRAW HEALTH BAR
	local percent = (self.health / self.maxhealth) * 100
	local lx, ly = sx - self.h, sy - self.h - 20
	local lx2, ly2 = sx + self.h, ly
	local dist = math.dist(lx, ly, lx2, ly2)
	local line_dist = dist * (percent/100)
	love.graphics.line(lx, ly, lx + line_dist, ly2)
	
	-- PRINT SELECTED WEAPON
	love.graphics.setColor(255,255,255)
	local font_w = mainFont:getWidth(self.weapon.weapon_type) -- not sure if this detects scale too. seems to return half as wide as i sould expect.
	love.graphics.print(self.weapon.weapon_type, sx - font_w , ly - 50, 0, 2, 2)
	

	if (game.settings.debug) then
		-- DEBUG TESTING
		local linex, liney
		linex = sx + 200 * math.cos(self.angle)
		liney = sy + 200 * math.sin(self.angle)
		love.graphics.line(sx, sy, linex, liney)  -- draw line to indicate enemy's current angle.
		
		-- DEBUG TESTING
		-- draw path tiles
		if self.path ~= nil then
			for node, count in self.path:nodes() do
				game:drawTileBorderRowCol(node.y, node.x)		  
				local nodex, nodey = world:getTileXY(node.y, node.x)
				if count == 2 then
					love.graphics.setColor(255, 207, 87)
				end
				love.graphics.circle('fill', nodex, nodey, 10) -- draw dots on tiles
				
			end
		end
		
	end
	
end

function Enemy2:update(sh, dt)
		local px, py = player1:center()
		local sx, sy = sh:center()			

		

		local LOS = game:checkLOS(sh, player1) -- check if we have line of sight to the player. only walls are considered.
		
		if LOS then -- move towards player. otherwise pathfind.
		
			local newangle = math.angle(sx, sy, px, py)
			
			-- determine shortest distance required to turn to newangle
			-- use degrees 
			local rotation_direction 
			if (math.deg(self.angle)-math.deg(newangle)+360) % 360 > 180 then 
				rotation_direction = "positive"
			else
				rotation_direction = "negative"
			end
			-- rotate in the appropriate direction			
			if rotation_direction == "positive" then
				self.angle = self.angle + self.rotatespeed *  dt
			elseif rotation_direction == "negative" then
				self.angle = self.angle - self.rotatespeed * dt
			end
			
			self:updateMovement(sh, dt)
		
		else -- NO LOS
		
			-- PATHFIND!
			local row, col = world:getTilePosition(sx, sy, true) -- get tile position in grid
			local newpath = game:pathFind(row, col, px, py) -- TEMP TEMP TESTING
			if newpath ~= nil then
				self.path = newpath
				--print ('set new path')
			else
				--print('newpath is nil')
			end
			
			-- set direction of movement. forget acceleration, friction for now.
			if self.path ~= nil then
				local firstnode
				local nodecount = 0
				
				for node, count in self.path:nodes() do -- why do i have to iterate i just want the second one??
					if count == 2 then -- starts at 1
						firstnode = node
					end
					
					nodecount = nodecount + 1
				end
				
				if nodecount >= 2 then
					local nodex, nodey = world:getTileXY(firstnode.y, firstnode.x) 					
					local newangle = math.angle(sx, sy, nodex, nodey)
					self.angle = newangle
					self:updateMovement(sh, dt)
				end
			end		
		
		end -- end if(LOS)
			
		
			
		-- TRY TO FIRE WEAPON 
		self.weapon:update(dt) 
		local dist = math.dist(sx,sy,px,py) 		
		if (dist < self.max_fire_range) and (dist > self.min_fire_range) then
			if self.health > 0 then -- enemy shouldn't attack after it's dead (self.health > 0)
				self:attack(sh) 				
			end
		end
	
end
  
function Enemy2:updateMovement(sh, dt)
	local px, py = player1:center()
	local sx, sy = sh:center()	
	
	local dist = math.dist(sx,sy,px,py) 
	
	-- acceleration
	-- maybe stop accelerating when the shortest angle required to turn is >= 90deg? maybe not needed.
	-- stop accelerating as you approach a reasonable range from target (player)
	if (dist > self.max_fire_range / 2 ) then 
		self.accelerationX = self.accelerationX  + self.thrust * math.cos(self.angle);
		self.accelerationY = self.accelerationY + self.thrust * math.sin(self.angle);
		-- print ('accelerating!')
	else
		-- print ('stopped accelerating')
	end

	-- velocity
	self.speedX = self.speedX + self.accelerationX * dt
	self.speedY = self.speedY + self.accelerationY * dt

	--friction
	self.speedX = self.speedX * self.friction
	self.speedY = self.speedY * self.friction
	
	-- zero out every frame.
	self.accelerationX = 0
	self.accelerationY = 0
	
	if self.speedY > self.maxspeed then
		self.speedY = self.maxspeed
	elseif self.speedY < (self.maxspeed * -1) then
		self.speedY = (self.maxspeed * -1)
	end
	if self.speedX > self.maxspeed then
		self.speedX = self.maxspeed
	elseif self.speedX < (self.maxspeed * -1) then
		self.speedX = (self.maxspeed * -1)
	end
	
	sh:move(self.speedX * dt, self.speedY * dt)

end
  
function Enemy2:setColor()  

	local color = self:getColor()

	love.graphics.setColor(color.r, color.g, color.b)

end

function Enemy2:getColor()
	local color = {}

	color.r = 201
	color.g = 130
	color.b = 196

	return color
end
  
function Enemy2:getHit(sh, damage) 
	self.health = self.health - damage
end 
  
 function Enemy2:explode(sh)  --  shape	
	self.health = 0 -- ugh, he will explode in die() function.	
end
  
  
  
 function Enemy2:die(i, sh) -- index, shape
 
	player1.instance.score = player1.instance.score + self.pointvalue
 
	local x, y = sh:center()
	local size = self.h
	
	-- explode
	game:spawnExplosion(x, y, size, true)

	-- announce
	game:spawnAnnouncement(x - size / 2, y - size *2, "gold", tostring(self.pointvalue) .. " pts")
	
	
	
	-- remove enemy from table
	Collider:remove(sh)
	table.remove(entities, i)

	-- enemy drops a modifier pickup
	local newEntity = Entities("pickup", x, y)
	newEntity:spawn()
		
	
 
 end
 
 function Enemy2:resolveCollision(sh, dx, dy)

	-- move shape to resolve collision
	sh:move(dx, dy)
	
	-- rebound 
	if (math.abs(dx) > math.abs(dy)) then
		-- if you are moving in a direction, being hit in the same direction should not cause you to reverse direction
		if dx < 0 and self.speedX > 0 or dx > 0 and self.speedX < 0 then 
			self.speedX = -self.speedX
		end
	else
		if dy < 0 and self.speedY > 0 or dy > 0 and self.speedY < 0 then
			self.speedY = -self.speedY
		end
	end

end

function Enemy2:resolveRepel(sh, dx, dy, repel_shape) --
local force = repel_shape.instance.force
local sx, sy = sh:center()
local rx, ry = repel_shape:center()
local repel_angle = math.angle(rx, ry, sx, sy)

	-- move shape to resolve collision
	sh:move(dx, dy) 

	-- rebound 
	if (math.abs(dx) > math.abs(dy)) then
		-- if you are moving in a direction, being hit in the same direction should not cause you to reverse direction
		if dx < 0 and self.speedX > 0 or dx > 0 and self.speedX < 0 then 
			self.speedX = -self.speedX
		end
	else
		if dy < 0 and self.speedY > 0 or dy > 0 and self.speedY < 0 then
			self.speedY = -self.speedY
		end
	end
	
	
	self.speedX = self.speedX + force * math.cos(repel_angle)
	self.speedY = self.speedY + force * math.sin(repel_angle)

end


function Enemy2:attack(sh)

	
	local px, py = player1:center()
	local sx, sy = sh:center()
	
	local angle = math.angle(sx, sy, px, py)
	
	local points = {}
	points[0] = { x = sx, y = sy }
	self.weapon:fire(points, self.speedX, self.speedY, angle, self.name) -- FIRE
	
	
	
end
  
  
   
return Enemy2