local class = require 'lib.middleclass'
local gamera = require('lib.gamera')
local vector  = require('lib.hardoncollider.vector-light') -- vector lib / i removed LOS, shouldnt need currently

Game = class('Game')


function Game:initialize() --originating entity
	
	self.state = 'game' -- menu, gameover, etc? do something with this at some point.
	self.hiscore = 0
	self.level = 0
	self.cam1 = nil 	-- main camera
	
	self.settings = {}
	self.settings.debug = false
	self.settings.sidebar = true
	
	self.levelduration = 15 -- duration to reset timer with 
	self.leveltimer = self.levelduration -- level countdown timer until borders toggle to warp
	
	self.startseed = math.random(1,10000)
	
	self.pathfind_test = {}
	
	self.newhiscore = false
end


function Game:start()
	-- destroy player1 instance
	player1 = nil
	
	-- stop any sounds from previous game
	love.audio.stop()

	self.startseed = math.random(1,10000)
	math.randomseed(game.startseed + os.time())
	print("randomseed", game.startseed)
	
	-- globals
	Collider = HC(100, on_collide) -- hardonCollider instance, heh heh
	Collider:clear() -- clear all collider objects. 

	-- DEBUGGING 
	--[[
	local test =  Collider:addRectangle(0, 0, 10, 10)
	Collider:addToGroup("group_repel", test)	
	Collider:remove(test)
	test = nil
	print(test)
	Collider:removeFromGroup("group_repel", test) 
	]]--
	
	-- first initialization
	local rows = math.random(5,13) -- have to define this in world.lua also
	local cols = rows + 7
	
	bullets = {} 
	--enemies = {} -- removed
	explosions = {}	
	entities = {} -- enemes, pickups
	announcements = {} -- popup text like "score" "pickup" etc.
	
	-- init world
	
	world = World(5000, 3000, rows, cols)	--  (w,h,rows,cols)
	world:generate()
	
	-- player and entities
	local px, py = world.w/2, world.h - (20) 
	player1 = self:spawnPlayer(px, py,'player')
	sfx.thruster:play() -- start looping thruster sfx
	
	-- initialise level timer 
	self.leveltimer = self.levelduration
	
	
	for i, v in pairs(sfx.bgm) do
		sfx.bgm[i]:rewind()
	end
	
	local bgm_selection = math.random(0,#sfx.bgm)
	sfx.bgm[bgm_selection]:play() -- start BGM track 
	print('BGM selected: #' .. bgm_selection)
	
	-- REMOVE THIS 
	nme_spawn_speed = 10 -- TEMPORARY. set as game class property later.
	nme_spawn_timer = nme_spawn_speed
	
	--rest hi score flag
	self.newhiscore = false
	
	-- main camera
	local retainCamScale = nil
	if self.cam1 then
		retainCamScale = self.cam1:getScale()
	end
	self.cam1 = gamera.new(-world.bufferpixels/2,  -world.bufferpixels/2, world.w + world.bufferpixels, world.h + world.bufferpixels)
	if retainCamScale then
		self.cam1:setScale(retainCamScale)
	else
		self.cam1:setScale(0.4)
	end
	
	-- screen area
	local window_width, window_height, window_flags = love.window.getMode()
	if (self.settings.sidebar == true) then
		self.cam1:setWindow(10, 10, window_width - 240 - 20, window_height - 20) -- self.cam1:setWindow(10,10,1020,700)
	else
		self.cam1:setWindow(10, 10, window_width - 20, window_height - 20) -- self.cam1:setWindow(10,10,1260,700)
	end
	
	
	
	-- player input device
	inputDevice = Controller()
	
end

function Game:update(dt)

	self:updateBullets(dt)
	
	Entities:update(dt) -- replaced updateEnemies() 
	
	self:updateExplosions(dt)
	self:updateAnnouncements(dt)
	game:updateScore(player1.instance.score)
	

		game:updateSpawnCountdown(dt)
		game:updateLevelCountdown(dt)

	
	world:updateWorld(dt)

end

function Game:updateLevelCountdown(dt)

	if self.leveltimer < 0 then	
		world:toggleBorders()		
		-- play PING sound effect.
		sfx.ding:play()
	
	
		--- TEPORARILY REMOVED. This allows player to finish level before leaving.
		-- It makes a bit of sense now with the additional enemy spawning commented out (below)
--		Entities:resetAll()  
		
		self.leveltimer = 0
		
	elseif (self.leveltimer > 0) then
		self.leveltimer = self.leveltimer - dt
	end

end

function Game:changeVolume(amount)
	local new_volume = love.audio.getVolume() + amount
	
	if new_volume >= 0 and new_volume <= 1 then
		love.audio.setVolume(new_volume)
	end
	
end


function Game:updateSpawnCountdown(dt) -- REMOVED AUTO SPAWNER - TEST - TEMPORARY

	if (self.leveltimer > 0) then

			-- --auto spawn enemies
			-- local nme_count = 0
			-- for k,v in pairs(entities) do
				-- if v.type == "enemy1" then
					-- nme_count = nme_count + 1
				-- end
			-- end
			
			-- if nme_spawn_timer <= 0 then
				-- if nme_count < 40 then
					-- self:addEntity('enemy1')
					-- nme_spawn_timer = nme_spawn_speed
				-- end
			-- else
				-- --countdown spawn timer
				-- nme_spawn_timer = nme_spawn_timer - dt
			-- end
			
	end
		
end


function Game:updateScore(newscore)
		-- track score (TEMP)
		local modifier = game:getStageModifier() 
		local actualscore = newscore * modifier
		
end

function Game:updateExplosions(dt)
		--update explosions
		for i, v in ipairs(explosions) do
			v:updateExplosion(i, dt) -- Not a collider shape, just love.draw
		end
end

function Game:updateAnnouncements(dt)
	--update announcements
	for i, v in ipairs(announcements) do
		v:update(i, dt)
	end
end


-- move this into weapons class?
function Game:updateBullets(dt)

		--update bullets
		for i, v in ipairs(bullets) do
			v.instance:update(v, dt)	
			
			if (v.instance.lifespan <= 0) then
				v.instance:die(i, v)
			end
		end

end


function Game:spawnExplosion(x, y, radius, playSound) -- (bool)playSound

	local explo = Explosion(x, y, radius) 
	table.insert(explosions, explo)

	if playSound == true then
		if sfx.explosion:isPlaying() then
			local newSource = sfx.explosion:clone()
			newSource:play()
		else
			sfx.explosion:play()
		end		
	end
	
end

function Game:spawnAnnouncement(x, y, announce_type, announce_text)

	local announce = Announce(x, y, announce_type, announce_text)
	table.insert(announcements, announce)
	
	-- play sound? should already be playing something
end


function Game:spawnPlayer(x,y,name)

	-- instantiate player class
	local spawnedPlayer = Player(x, y, name)
	local p = spawnedPlayer.ship
	
	--initialize shape with HC
	local shape = Collider:addPolygon(p.x1 + x, p.y1 + y, p.x2 + x, p.y2 + y, p.x3 + x, p.y3 + y)
	-- assign basic properties
	shape.type = "player"
	shape.name = "player"
	
	-- assign instance of player class as property of shape. makes class functions accessible from on_collision()
	shape.instance = spawnedPlayer
	
	--rotate
	--shape:rotate(p.shipfacing)
	
	-- scale the ship?	
	shape:scale(p.xscale) -- scale in class draw function
	
	-- return shape with accessible class functions
	return shape
end


-- move this into weapons class?
function Game:drawProjectiles()
	--draw bullets
	for i, v in ipairs(bullets) do
		v.instance:draw(v)
	end
end

function Game:drawExplosions()
		for i, v in ipairs(explosions) do
			v:drawExplosion()
		end
end

function Game:drawAnnouncements()
	for i, v in ipairs(announcements) do
		v:draw()
	end
end

function Game:drawTileBorderXY(x, y, isGlobalCoord)

		local w = math.round(world.w / world.columns) -- 1 column width
		local h = math.round(world.h / world.rows) -- 1 row width
		local trow, tcol = world:getTilePosition(x, y, isGlobalCoord)
		
		local tilex = tcol*w + world.left
		local tiley = trow*h + world.top
		
		love.graphics.setColor(85,77,179,80)
		love.graphics.rectangle('line', tilex, tiley, w, h)	

end

function Game:drawTileBorderRowCol(row, col) --

		local w = math.round(world.w / world.columns) -- 1 column width
		local h = math.round(world.h / world.rows) -- 1 row width
		
		
		local tilex = col*w + world.left
		local tiley = row*h + world.top
		
		love.graphics.setColor(85,77,179,80)
		love.graphics.rectangle('line', tilex, tiley, w, h)	

end



function Game:addEntity(entity_type)

	local x, y
	local px, py = world.w/2, world.h - (20) -- if player is not initialized, it is the first zone, use player starting spawn as x,y
			
	-- if player is initialized, user player1 instance values.
	if (player1 ~= nil) then  -- i don't like having to do these checks
		px, py = player1:center() 
	end
	
	repeat
		x = math.random(110,world.w - 110) 
		y = math.random(110, world.h - 110) 
		
		local trow, tcol = world:getTilePosition(x, y, false)
		--print(trow, tcol)
		local tile = world:getTileValue(trow, tcol) 
		if (tile == "#") then
			--print ( "cant' spawn in wall" )
		end

		
		local pxq, pyq = 0,0
		if (player1 ~= nil) then -- i don't like having to do these checks
			pxq = player1.instance.xquadrant
			pyq = player1.instance.yquadrant
		end
		
		--convert enemy x,y to full grid x,y for math.dist() comparison with player (who is already spawned).
		e_grid_x = x + ((pxq) * (world.w + world.bufferpixels)) 
		e_grid_y = y + ((pyq) * (world.h + world.bufferpixels))  
		
	
	until (math.dist(e_grid_x, e_grid_y, px, py) > 800) and (tile ~= "#") -- spawn away from player & not on wall

	--print("PX, PY for spawning : ", px, py)
	--print (entity_type, e_grid_x, e_grid_y, px, py)
	
	local newEntity = Entities(entity_type, x+world.left, y+world.top)
	newEntity:spawn()

end


function Game:pathFind(startrow, startcol, x, y)

		local walkable = function(value) 
									if (value == ' ' or value == 'R') then -- gonna have to create a mapping table or something
										return true
									end
									return false
								end

	
		-- Calls the grid class
		local Grid = jumper_grid
		-- Calls the pathfinder class
		local Pathfinder = jumper_pf

		-- Creates a grid object
		local grid = Grid(world.level_tiles) --world.level_tiles)

		-- Creates a pathfinder object using A* Search algorithm
		local myFinder = Pathfinder(grid, 'ASTAR', walkable)
		myFinder:setMode('DIAGONAL') -- ORTHOGONAL || DIAGONAL
	
		--local px, py = player1:center()
		--print(startrow, startcol, x, y)
		
		local row, col = world:getTilePosition(x, y, true) -- get tile position in grid
		local endcol, endrow = col, row -- -- assign them appropriately in this context col = x, row = y.
				
		local path = nil
		
		-- don't pathfind if the parameters are out of bounds
		if startcol >= 0 and startcol <= world.columns and startrow >= 0 and startrow <= world.rows then
			if endcol >= 0 and endcol <= world.columns and endrow >= 0 and endrow <= world.rows then
				-- Calculates the path, and its length
				-- reverse the x, y tile coordinates to make "col, row"
				path = myFinder:getPath(startcol, startrow, endcol, endrow, false) -- startcol, startrow, endcol, endrow, (TUNNEL = false)
				--print('got path')
				if path == nil then
					--print('path not found')
				end				
			else			
				--print('failed on endrow, endcol' .. endrow, endcol)
			end
		else 
			--print('failed on startrow, startcol' .. startrow, startcol)
		end
		
		--print(startcol, startrow, endcol, endrow)
		
	return path
end


function Game:checkLOS(shape, shape2) -- only checks WALLS for LOS. shape2 i expect should always be player1

	local hasLineOfSight = true
	
	-- RAYCAST towards player
	local px, py = shape2:center() -- player x,y
	local sx, sy = shape:center() -- enemy x,y
	
	for k, j in ipairs(world.entities.walls) do -- need to skip the first 4 walls/borders?
			--local wallx, wally = j:center()
			
			local dx = px - sx
			local dy = py - sy
			
			love.graphics.setLineWidth(1)
			love.graphics.setColor(255,0,0)
			-- Draw line?
			
			local intersecting, t = j:intersectsRay(sx, sy, dx, dy)--destx, desty)
			
			if intersecting then		
					-- find point of intersection
					local vx,vy = vector.add(sx, sy, vector.mul(t, dx, dy))
					local distance_source_to_player = math.dist(sx, sy, px, py)
					local distance_source_to_intersect_shape = math.dist(sx, sy, vx, vy)								
					
					-- if angle to player is same as angle to wall then we have found a wall between 
					-- the enemy and the player position.
					--must round these numbers or you get flicker.
					local angle_source_to_player = math.round(math.angle(sx, sy, px, py))
					local angle_source_to_intersection =  math.round(math.angle(sx, sy, vx, vy))
					--print (angle_source_to_player, angle_source_to_intersection)
					
					-- no line of sight.
					if angle_source_to_player == angle_source_to_intersection then								
						if (distance_source_to_player > distance_source_to_intersect_shape) then
							hasLineOfSight = false
						end
						
					end															
			end
	end
				
	return hasLineOfSight
end


function Game:getStageModifier()
	
	if player1 ~= nil then
		return (math.abs(player1.instance.xquadrant) + math.abs(player1.instance.yquadrant)) + 1
	else
		return 1
	end
	
end

return Game