local class = require 'lib.middleclass'
-- require 'bullet'


Player = class('Player')

function Player:initialize(x, y, name) 

 -- load from config later...

	self.name = name	
	
	self.ship =	{		
							x1 = 0,  
							y1 = 0, 
							x2 = 100, 
							y2 = 0, 
							x3 = 50, 
							y3 =  -200, 
							xscale = .3,  -- are both of these necessary
							yscale = .3,  -- are both of these necessary
							angle = math.rad(-90), --
							rotatespeed = 4.5, 
							shipfacing = math.rad(-90),
							speedX = 1 ,
							speedY = 1,
							accelerationX = 0,
							accelerationY = 0,
							maxspeed = 500,
							thrust = 550,
							friction = .995,
							alive = true,
							invuln = { state = 'off', timer = 0, timermax = 0.5 }, -- for temporary invulnerability
							thruster = 	{ state = 'off'	},
							afterburner = {state = 'off' },
							shield = { state = 'off', timer = 3, timermax = 3, armed = true},
							guns = { state = 'dual' },
							repel = { count = 5, maxcount = 5 },
							weaponguns = 'bullet',
							weaponbomb = 'bomb',
							weaponrepel = 'repel'
						}
	self.xquadrant = 0
	self.yquadrant = 0
	
	self.score = 0
	
	self.maxhealth = 100
	self.health = self.maxhealth
	
	-- if adding another one of these don't forget to call weapontype:update in player:update...
	self.weaponguns = Weapon(self.ship.weaponguns, self.name)
	self.weaponguns.armed = true
	self.weaponbomb = Weapon(self.ship.weaponbomb, self.name)
	self.weaponbomb.armed = true
	self.weaponrepel = Weapon(self.ship.weaponrepel, self.name)
	self.weaponrepel.firingtimer = 0.5
	self.weaponrepel.armed = true
							
end

function Player:updateShield(dt)
	if self.ship.shield.armed == true then
		if self.ship.shield.timer >= self.ship.shield.timermax and self.ship.shield.state == 'off' then
			self.ship.shield.state = 'on'
		elseif self.ship.shield.state == 'off' then
			self.ship.shield.timer = self.ship.shield.timer + dt
		else 
			self.ship.shield.timer = 0
		end
	end
end

function Player:updateInvuln(sh, dt)
	
		if self.ship.invuln.timer >= self.ship.invuln.timermax and self.ship.invuln.state == 'on' then
			self.ship.invuln.state = 'off'
			self.ship.invuln.timer = 0
		elseif self.ship.invuln.state == 'on' then
			self.ship.invuln.timer = self.ship.invuln.timer + dt
		else 
			self.ship.invuln.timer = 0
		end
	
end


function Player:update(sh, dt) -- shape, delta-time
	
	local p = self.ship
	
	self:checkKeys(sh, dt)
	self.weaponguns:update(dt)
	self.weaponbomb:update(dt)
	self.weaponrepel:update(dt)
	self:updateShield(dt)
	self:updateInvuln(sh, dt)

	
	-- play thruster SFX
	if p.thruster.state == 'forward' or p.thruster.state == 'reverse' then
		if sfx.thruster:isPlaying() == true then
			sfx.thruster:setVolume(0.5) -- turn up volume of thruster
		end
	else
		if sfx.thruster:isPlaying() == true then
			sfx.thruster:setVolume(0.2) -- turn down volume of thruster
		end
	end	

	--velocity
	p.speedX = p.speedX + p.accelerationX * dt
	p.speedY = p.speedY + p.accelerationY * dt

	--friction
	p.speedX = p.speedX * p.friction
	p.speedY = p.speedY * p.friction

	--zero out acceleration each frame
	p.accelerationX = 0
	p.accelerationY = 0
		
	local maxspeed = p.maxspeed
	if p.afterburner.state == 'on' then
		sfx.thruster:setVolume(0.7) -- turn up volume of thruster
		maxspeed = maxspeed * 1.5
	
	end
		
	--maxspeed. I swear i can go faster diagonally.
	--stopping afterburner creates a kind of hard slowdown in speed. I would like it to be less of a jolt.
	if p.speedX > maxspeed then
		p.speedX = maxspeed
	elseif p.speedX < (maxspeed * -1) then
		p.speedX = (maxspeed * -1)
	end

	if p.speedY > maxspeed then
		p.speedY = maxspeed
	elseif p.speedY < (maxspeed * -1) then
		p.speedY = (maxspeed * -1)
	end
	
	-- move shape coordinates
	sh:move(p.speedX * dt, p.speedY * dt)	
		
end

function Player:draw(sh) -- sh - player1's collider:shape
	local p = sh.instance.ship
	local sx, sy = sh:center()
	
	local shipColor = {r = 85, g = 77, b = 179}
	
	if self.ship.invuln.state == 'on' then 
		shipColor = {r = 62, g = 33, b = 64}
	end
	
	
	-- draw thruster


	love.graphics.push()

		love.graphics.translate(sx,sy)	
		love.graphics.scale(p.xscale)
		love.graphics.rotate(p.shipfacing) -- THIS IS WHY ITS SIDEWAYS. I SET math.rad(-90). gotta fix.
		--love.graphics.rotate(p.shipfacing + math.rad(90)) -- this would fix it, but now i dont want to.
		
		
		-- DRAW DUAL GUNS
		-- pretend the ship is drawn facing right >>>
		if (p.guns.state == 'dual') then
			-- weapon anchor...
			love.graphics.setColor(shipColor.r, shipColor.g, shipColor.b , shipColor.a)
			love.graphics.rectangle('fill', 10, -60, 20, 120)
			-- left gun
			love.graphics.setColor(255,255,0)
			love.graphics.rectangle('fill', 10, -60 - 20, 60, 20)
			-- right gun
			love.graphics.setColor(255,255,0)
			love.graphics.rectangle('fill', 10, 60, 60, 20)
			
		end
		
		
		-- thruster state
		if p.thruster.state == 'forward' then
			love.graphics.setColor(255,0,0,90)
		elseif p.thruster.state == 'reverse' then
			love.graphics.setColor(0,0,255,90)
		else
			love.graphics.setColor(85,77,179, 0)	
		end
			
		
		-- DRAW THRUSTER
		-- pretend the ship is drawn facing right >>> it makes more sense.
		local tw, th = 50, 75
		if p.afterburner.state == 'on' then
			tw = tw * 2
		end
		love.graphics.rectangle('fill', 0 - tw/2 - 75, 0 - th/2, tw, th) 
	
		
	love.graphics.pop()

	-- SHIELD
	
	-- draw shield timer bar
	love.graphics.setColor(79, 124, 176) 
	love.graphics.setLineWidth(2)
	local percent = (self.ship.shield.timer / self.ship.shield.timermax) * 100
	local lx, ly = sx - 50, sy + 50
	local lx2, ly2 = sx + 50, ly
	local dist = math.dist(lx, ly, lx2, ly2)
	local line_dist = dist * (percent/100)
	love.graphics.line(lx, ly, lx + line_dist, ly2)
	
	if self.ship.shield.state == 'on' then	
		-- draw shield
		love.graphics.setColor(79, 124, 176, 50) 
		love.graphics.circle('fill', sx, sy, 75)
		
	end	
	love.graphics.setLineWidth(1)
		
		
	-- draw invuln timer bar
	love.graphics.setColor(255, 207, 87) 
	love.graphics.setLineWidth(2)
	local percent = (self.ship.invuln.timer / self.ship.invuln.timermax) * 100
	local lx, ly = sx - 50, sy + 60
	local lx2, ly2 = sx + 50, ly
	local dist = math.dist(lx, ly, lx2, ly2)
	local line_dist = dist * (percent/100)
	love.graphics.line(lx, ly, lx + line_dist, ly2)

		
	-- SHIP BODY
	love.graphics.setColor(shipColor.r, shipColor.g, shipColor.b, shipColor.a)
	sh:draw('fill')	

	
	-- DISPLAY ACTIVE TILE AREA
	if (game.settings.debug) then		
		game:drawTileBorderXY(sx, sy, true)
	end
	
end

-- FIRE
function Player:fire(sh, weapontype) -- shape, (future parameter:weapon would allow selection of different projectile class)
	
	local p = self.ship
	local px, py = sh:center() -- get center of ship that fired
	
	--NEW PLAN
	-- Imagine the ship is drawn facing right > (or i can set p.shipfacing + math.rad(90))
	local a = p.shipfacing
	local cosa = math.cos(a)
	local sina =  math.sin(a)
	
	
	-- center gun (default)
	local midx, midy = px + 50, py
	local x = px + (midx - px) * cosa - (midy - py) * sina
	local y = py + (midx - px) * sina + (midy - py) * cosa
	

	-- left gun
	local leftx = px + 25
	local lefty = py - 20
	local x2 = px + (leftx - px) * cosa - (lefty - py) * sina
	local y2 = py + (leftx - px) * sina + (lefty - py) * cosa
		
	-- right gun
	local leftx = px + 25
	local lefty = py + 20
	local x3 = px + (leftx - px) * cosa - (lefty - py) * sina
	local y3 = py + (leftx - px) * sina + (lefty - py) * cosa
		
			
	local points = {}
	if weapontype.weapon_type == 'bullet' then
		if (p.guns.state == 'dual')	then
			points[0] = {x = x2, y = y2}
			points[1] = {x = x3, y = y3}
		else
			points[0] = {x = x, y = y}  -- default center gun.
		end
	else
		points[0] = {x = x, y = y} -- bomb/repel currently.
	end


	weapontype:fire(points, p.speedX, p.speedY, p.shipfacing, self.name) -- FIRE
	
end

function Player:getHit(sh, colliding_shape) -- sh = self shape, colliding_shape (shape that collided with you)
	local px, py = player1:center()
	local shieldDamage = game:getStageModifier() * 100 -- should always kill enemy1/enemy2 atm, but NOT boss1
	local endgame = false

	
	if self.ship.invuln.state  == 'off' and self.health > 0 then -- don't take damage while invuln? should still bounce around though.
	
		if self.ship.shield.state == 'on' then
		
			--print('shield hit. shape-id:  ' .. colliding_shape.instance.id)
			-- setghost should not be necessary with invuln timer now.
			--Collider:setGhost(colliding_shape) -- maybe this will prevent dying to an instant follow up collision before the shape is removed.
			colliding_shape.instance:getHit(colliding_shape, shieldDamage) -- deal damage to colliding shape
			--print ('trigger colliding_shape.instance:getHit()')
			self.ship.shield.state = 'off' -- disable shield ~ ALWAYS DO THIS LAST~
			--print ('shield off')
		else
			--print('regular hit. shape-id: ' ..  colliding_shape.instance.id)
			
			
			if colliding_shape.instance.damage ~= nil then
				
				self.health = self.health - colliding_shape.instance.damage
				
				-- Announce Damage
				local msg = "-" .. tostring(colliding_shape.instance.damage) .. " health"
				local font_w = mainFont:getWidth(msg) -- not sure if this detects scale too. seems to return half as wide as i sould expect.
				game:spawnAnnouncement(px - font_w, py - 40, "red", msg)
				
			else
			
				self.health = self.health - 25 -- default damage value for collisions with shapes that don't have a damage property.
				
				-- Announce Damage
				local msg = "-25 health"
				local font_w = mainFont:getWidth(msg) -- not sure if this detects scale too. seems to return half as wide as i sould expect.
				game:spawnAnnouncement(px - font_w, py - 40, "red", msg)
				
			end
			
			self.ship.invuln.state = 'on'
		
		end
		
	end

	if self.health <= 0 then
		endgame = true
	end
	
	
	-- GAME OVER
	if endgame == true then
	
		love.audio.stop()
		sfx.explosion:play()
		
		
		
		
		if (player1.instance.score > game.hiscore) then -- user set high score.. 
			sfx.cheer:play()			
			game.hiscore = player1.instance.score
			game.newhiscore = true --flag hi score
		end
		
		-- stop everything. except pickups from spinning?
		self.ship.alive = false
		game.state =  'gameover'
	
	end
		
end

function Player:warp(sh, direction)
local px, py = sh:center()

local warp_distance = (world.bufferpixels) + (math.abs(sh.instance.ship.y3)*sh.instance.ship.xscale) + (20*2) -- plus border width * 2


	if (direction == "left") then
		sh:move(-warp_distance, 0)
		self.xquadrant = self.xquadrant - 1
	elseif (direction == "top") then
		sh:move(0, -warp_distance)
		self.yquadrant = self.yquadrant - 1
	elseif (direction == "right") then
		sh:move(warp_distance, 0)
		self.xquadrant = self.xquadrant + 1
	elseif (direction == "bottom") then
		sh:move(0, warp_distance)
		self.yquadrant = self.yquadrant + 1
	end

	
	--print("UPDATED X,Y QUADRANT! : ", self.xquadrant, self.yquadrant)
	--print("INSTANCE X,Y QUADRANT? : ", player1.instance.xquadrant, player1.instance.yquadrant)

	-- stop player after warp so they dont bounce off something 
	self.ship.speedX = 0
	self.ship.speedY = 0

	sfx.door_slam:play()
	sfx.siren:play()
	
	world:toggleBorders() -- switch borders to warps
	
	--reset level timer && increase duration based on largest quadrant value. (ooh ternary oprations)
	local largest_quadrant = (math.abs(self.xquadrant) > math.abs(self.yquadrant) and math.abs(self.xquadrant) or math.abs(self.yquadrant)) 
	
	--print("largest quadrant of", self.xquadrant, self.yquadrant, "is : ", largest_quadrant)
	
	game.leveltimer = game.levelduration + (5 * largest_quadrant) 
	
	
end


function Player:resolveCollision(sh, dx, dy)

	-- move shape to resolve collision
	sh:move(dx, dy)
	
	-- rebound 
	if (math.abs(dx) > math.abs(dy)) then
		if dx < 0 and self.ship.speedX > 0 or dx > 0 and self.ship.speedX < 0 then
			self.ship.speedX = -self.ship.speedX
		end
	else
		if dy < 0 and self.ship.speedY > 0 or dy > 0 and self.ship.speedY < 0 then
			self.ship.speedY = -self.ship.speedY
		end
	end

	if sfx.bounce:isPlaying() then
		local newSource = sfx.bounce:clone()
		newSource:play()
	else
		sfx.bounce:play()
	end
	
end

function Player:checkKeys(sh, dt)
	inputDevice:checkKeyboard(sh, dt)	
	-- inputDevice:checkJoystick(sh,dt)	 -- not fully functional. causes a sound bug too.
end



function Player:reverseThrust(sh, dt)
	local p = sh.instance.ship
		
	p.angle = p.shipfacing
	p.thruster.state = 'reverse'

	p.accelerationX = p.accelerationX  - p.thrust * math.cos(p.angle);
	p.accelerationY = p.accelerationY - p.thrust * math.sin(p.angle);
end

function Player:forwardThrust(sh, dt)
	local p = sh.instance.ship
	local thrust = p.thrust
	
	if p.afterburner.state == 'on' then
		thrust = thrust * 2
	end
	
	p.angle = p.shipfacing
	p.thruster.state = 'forward'

	p.accelerationX = p.accelerationX  + thrust * math.cos(p.angle);
	p.accelerationY = p.accelerationY + thrust * math.sin(p.angle);
end

function Player:rotateLeft(sh, dt)
	local p = sh.instance.ship

	p.shipfacing = p.shipfacing - p.rotatespeed * dt
	sh:rotate(-p.rotatespeed * dt)
end

function Player:rotateRight(sh, dt)
	local p = sh.instance.ship

	p.shipfacing = p.shipfacing + p.rotatespeed * dt
	sh:rotate(p.rotatespeed * dt)
end

function Player:afterburner()

end



return Player