local class = require 'lib.middleclass'

Controller = class('Controller')

function Controller:initialize() --originating entity
	
	local joysticks = love.joystick.getJoysticks()
	if joysticks then
		self.joystick = joysticks[1]
	end
		
		self.keyboard_map = {
							up = "up",
							down = "down",
							left = "left",
							right = "right",
							bullet = " ",
							bomb = "tab",
							cam1 = "8",
							cam2 = "9",
							cam3 = "0",
							vol_up = "x",
							vol_down = "z",
							toggle_sidebar = "f",
							afterburn = 'lshift', 
							repel_btn1 = "tab",
							repel_btn2 = "lshift", 
							
							d_bug = "2",
							d_zoom_out = "s",
							d_zoom_in = "w",
							d_spawn_nme = "1",
							d_get_tile = "f1",
							d_add_buffs = "5"							
						}
	
	
	-- super basic gamepad binds. it's so hard to play on a pad...
	self.gamepad_map =  { 
							up =  "a", 
							down = "b",
							--rotate = "leftx", 
							left = "dpleft",
							right = "dpright",
							bullet = "rightshoulder",
							bomb = "leftshoulder",
							repel_btn1 = "triggerleft",
							repel_btn2 = "triggerright"
							

							--[[,
							cam1 = "8",
							cam2 = "9",
							cam3 = "0",
							
							d_bug = "f2",
							d_zoom_out = "s",
							d_zoom_in = "w",
							d_spawn_nme = "1",
							d_get_tile = "f1",
							d_add_buffs = "f5"	]]-- still using keyboard for these other things
						}
	
		
		
end

function Controller:update(dt)

	-- check axises!

end


function Controller:checkKeyboard(sh, dt)

	local keys = self.keyboard_map
	
	if isDown(keys.down) then  -- accelerate backwards
		sh.instance:reverseThrust(sh, dt)	
	elseif isDown(keys.up) then  -- accelerate forward
		sh.instance:forwardThrust(sh, dt)
	else
		sh.instance.ship.thruster.state = 'off' -- HMMMM
	end

	-- rotate left
	if isDown(keys.left) then 
		sh.instance:rotateLeft(sh, dt)
	end

	-- rotate right
	if isDown(keys.right) then 
		sh.instance:rotateRight(sh, dt)
	end
	
	-- shoot bullet
	if isDown(keys.bullet) then
		sh.instance:fire(sh, sh.instance.weaponguns)
	end
	-- shoot bomb
	if isDown(keys.bomb) and not isDown(keys.repel_btn2) then
		sh.instance:fire(sh, sh.instance.weaponbomb)
	end
	
	-- afterburner 
	if isDown(keys.afterburn) then
		sh.instance.ship.afterburner.state = 'on'
	else
		sh.instance.ship.afterburner.state = 'off'
	end
	
	-- repel
	if isDown(keys.repel_btn1) and isDown(keys.repel_btn2) then
		
		if sh.instance.weaponrepel.armed == true then
			sh.instance:fire(sh, sh.instance.weaponrepel)
		end
		
	end
	
	
	if isDown(keys.vol_up) then
		game:changeVolume(0.01)
	end
	
	if isDown(keys.vol_down) then
		game:changeVolume(-0.01)
	end
	
	-- RANDOM NOT REALLY PLAYER RELATED SHIT BELOW.
	
	--add enemy
	if isDown(keys.d_spawn_nme) then
		if (game.settings.debug) then
			--game:addEntity('enemy1')		
			--game:addEntity('boss1')		
			game:addEntity('pickup')
		end
	end
	
	-- ******* DEBUG ************* (this is better now that i cleaned up modifier code)
	if isDown(keys.d_add_buffs) and game.settings.debug then -- for testing of random modifiers
		
		
		-- select weapon for random bonus
		local g = math.random (1,2)
		local w
		local buffs		
		local r = math.random(1,5) -- random select buff
		
		local mod_props = WeaponModifiers:getPropNames()
		
		if g == 1 then 
			w = player1.instance.weaponguns
			w.modifiers:apply(w.weapon_type, mod_props[r])
			
		 elseif g == 2 then
			w = player1.instance.weaponbomb
			w.modifiers:apply(w.weapon_type, mod_props[r])
			 
		 end

		
	end
	-- *******END DEBUG **************

end


function Controller:checkJoystick(sh, dt)

	local joy1 = self.joystick
	local keys = self.gamepad_map
	
	if joy1 ~= nil then -- add keyboard/joystick toggle. thruster.state = 'off' is firing constantly if a device is plugged in while using keyboard.
	
		--local leftstick = joy1:getGamepadAxis(self.gamepad_map.rotate)
		--local rightstick = joy1:getGamepadAxis(self.gamepad_map.rotate)
		local lefttrigger = joy1:getGamepadAxis(self.gamepad_map.repel_btn1)
		local righttrigger = joy1:getGamepadAxis(self.gamepad_map.repel_btn2)
		
		
		if joy1:isGamepadDown(keys.up) then  -- accelerate backwards
			sh.instance:forwardThrust(sh, dt)	
		elseif joy1:isGamepadDown(keys.down) then  -- accelerate forward
			sh.instance:reverseThrust(sh, dt)
		else
			sh.instance.ship.thruster.state = 'off' -- HMMMM
		end

		-- rotate left
		if joy1:isGamepadDown(keys.left) then 
			sh.instance:rotateLeft(sh, dt)
		end

		-- rotate right
		if joy1:isGamepadDown(keys.right) then
			sh.instance:rotateRight(sh, dt)
		end
		
		-- shoot bullet
		if joy1:isGamepadDown(keys.bullet) then
			sh.instance:fire(sh, sh.instance.weaponguns)
		end
		
		if joy1:isGamepadDown(keys.bomb) then
			sh.instance:fire(sh, sh.instance.weaponbomb)
		end

		if lefttrigger > 0 and righttrigger > 0 then
			sh.instance:fire(sh, sh.instance.weaponrepel)
		end
		
		
	end
	
	
	
	
end


-- blah do something with this later.
function love.keyreleased(key)
	
	local keys = inputDevice.keyboard_map
	
	-- this always needs to be available.
	if key == 'escape' then
		game:start()
	end
	
	if key == keys.d_get_tile then
		local x, y = player1:center()
		local row, col = world:getTilePosition(x,y, true) -- pass true flag to convert full grid x,y to relative to world boundaries.
		print ('row, col' .. row, col)
		print('tile value : ' .. world:getTileValue(row, col))
		
		local tilex, tiley = world:getTileXY(row, col)
		print('tile x, y : ' .. tilex, tiley )
	end
	
	if key == keys.d_bug then
		-- toggle debug
		if (game.settings.debug == true) then
			game.settings.debug = false
		else
			game.settings.debug = true
		end
	end

	if key == keys.cam1 then
		game.cam1:setScale(0.5)
	end
	
	if key == keys.cam2 then
		game.cam1:setScale(0.4)
	end
	
	if key == keys.cam3 then
		game.cam1:setScale(0.3)
	end
	
	if key == keys.toggle_sidebar then 
		local window_width, window_height, window_flags = love.window.getMode()
		
		if (game.settings.sidebar == true) then
			game.settings.sidebar = false
			game.cam1:setWindow(10,10, window_width - 20, window_height - 20) -- game.cam1:setWindow(10,10,1260,700)
		else
			game.settings.sidebar = true -- window_width - sidebar_width - buffer
			game.cam1:setWindow(10,10,window_width - 240 - 20, window_height - 20) -- game.cam1:setWindow(10,10,1020, 700)
		end
	end
	
		
end

function love.resize(window_width, window_height) 
		if (game.settings.sidebar == false) then
			game.cam1:setWindow(10,10, window_width - 20, window_height - 20) -- game.cam1:setWindow(10,10,1260,700)
		else
			game.cam1:setWindow(10,10,window_width - 240 - 20, window_height - 20) -- game.cam1:setWindow(10,10,1020, 700)
		end
end

return Controller