
local class = require 'lib.middleclass'

Weapon = class('Weapon')

function Weapon:initialize(weapon_type, origin_entity, weapon_modifiers)

	self.weapon_type = weapon_type
	self.origin = origin_entity
	--self.armed = true
	
	--weapon properties	
	self.firingtimer = 1 -- first shot from weapon instantiation will have 1 second timer.
	
	-- I can use these properties to adjust ENEMY origin weapon values too which is nice.
	if weapon_modifiers then
		self.modifiers = weapon_modifiers -- init with preset modifiers.
	else
		self.modifiers = WeaponModifiers() -- init with no/default modifiers
	end
	 
end

function Weapon:fire(points, speedx, speedy, angle) -- points = { 0 = {x1, y1 }, 1 = { x2, y2 } ...}
local shot, shape

	local x, y = points[0].x, points[0].y -- first x,y position for projectiles to start 

	--print ('firing', self.firingdelay, self.firingtimer)
	
	if self.firingtimer <= 0 then	
	
	
	
		if (self.weapon_type == 'bullet') then			
			
			--support dual guns automatically
			for _, v in pairs(points) do
				shot = Bullet(v.x, v.y, speedx, speedy, angle, self.origin, self.modifiers)
				shape = Collider:addCircle(v.x, v.y, shot.radius)
				self:createProjectileInstance(shot, shape, "group_bullets", "bullet", "bullet")
			end
			
		elseif (self.weapon_type == 'bomb') then
			shot = Bomb(x, y, speedx, speedy, angle, self.origin, self.modifiers)
			shape = Collider:addCircle(x, y, shot.radius)	
		
			self:createProjectileInstance(shot, shape, "group_bullets", "bullet", "bullet")
		
		elseif (self.weapon_type == 'spreadbomb') then
			
			local spread_angle = 0
			for i = 0, 7 do
				spread_angle = math.rad(i * 45)
				shot = Bomb(x, y, speedx, speedy, spread_angle, self.origin, self.modifiers)
				shape = Collider:addCircle(x, y, shot.radius)	
		
				self:createProjectileInstance(shot, shape, "group_bullets", "bullet", "bullet")
			end
		
		elseif (self.weapon_type == 'repel' ) then
		
			shot = Repel(x, y, self.origin)
			shape = Collider:addCircle(x, y, shot.radius)
			Collider:setPassive(shape) -- set passive so it won't collide with stage geometry (walls, borders).
			
			self:createProjectileInstance(shot, shape, "group_repel", "repel", "repel")
			
			-- reduce stock
			player1.instance.ship.repel.count = player1.instance.ship.repel.count - 1
			-- de-arm weapon if 0 or below.
			if (player1.instance.ship.repel.count <= 0) then
				player1.instance.weaponrepel.armed = false
			end
		
		end
	
		
		
		-- play sound effect. move this into weapon class at some point.
		shot:playSound()
		
		-- set firingtimer to equivalent projectile value for next shot.
		-- next shot should include any modifiers applied to this current shot (and so on).
		self.firingtimer = shot.firingtimer 
		
	else
	
		--print('cant attack, countdown at : ', self.firingtimer)
		
	end
		
end

function Weapon:createProjectileInstance(shot, shape, group_name, shot_type, shot_name)

		shape.type = shot_type
		shape.name = shot_name
		Collider:addToGroup(group_name,shape)
		
		shape.instance = shot
		
		table.insert( bullets, shape)	-- INSERT INTO BULLETS TABLE

end

function Weapon:update(dt)

	if self.armed == true then -- if weapon is armed, start timer
	--firing delay timer
		if self.firingtimer < 0 then
			self.firingttimer = 0
		elseif self.firingtimer > 0 then
			self.firingtimer = self.firingtimer - dt 
		end
	
	end
end



return Weapon